c
c
c         DSP control program for Magellan
c         guider camera clock service routine.
c         ------------------------------------
c
       subroutine dspchk
c
       implicit none
c
       integer*4 step, i4, itry, dspsend
       integer*4 fifoct, fifchk, ttime, nbuf
       integer*4 dtime, ier, dspget, i, ibuf
       integer*4 dspini, dsperr, pcibuf (1024)
       integer*4 dspflg (47), dspval (47)
       integer*4 iobase, pt1map, pt2map
       integer*4 tmperr, tmpval, vbsval
       integer*4 xfsflg, exftim, exstim
       integer*4 exptim, runtim, acqtim
       integer*4 avp, avp1, avpf, avps
       integer*4 imgsiz, imgptr, imgflg, imgprv
       integer*4 nlo, pctval, dval, pctset, pctprv
       integer*4 pctvlf, pctvls, mbox, inpd
       integer*4 imgbuf (300000)
       logical*4 pctflg, upflg, prflg, cross, prvx
       logical*4 ampfl, dacfl
c
       integer*2 i2 (2)
c
       common / dspcom / dspini, dsperr, dspflg, dspval
       common / tmpcom / tmperr, tmpval, vbsval
       common / imgcom / imgsiz, imgptr, imgflg, imgbuf
       common / pcicom / iobase, pt1map, pt2map
       common / xfscom / xfsflg, exftim, exstim
       common / expcom / exptim, runtim, acqtim
       common / avpcom / avpf, avps
       common / pctcom / pctset, pctval, pctprv
c
       equivalence (i4, i2 (1))
c
       data  step / -1 /, nbuf / 655 /
       data  dval / 256 /
c
c
c         Don't do anything unless the DSP is running
c         properly.  Once it has been initialized, the
c         flag DSPINI is used to indicate that the DSP
c         is OK, or to indicate one of several possible
c         errors:
c
c            DSPINI = 2;   DSP OK.
c            DSPINI = 3;   FIFO count error.
c            DSPINI = 4;   DSP read-back time-out.
c            DSPINI = 5;   Parameter read-back error.
c            DSPINI = 6;   FIFO counter sync error.
c            DSPINI = 7;   DSP data time-out.
c
c         The variable DSPERR contains additional info
c         about an error condition.
c
c         Set the value of runtim to a big number so an
c         exposure will start as soon as the DSP is
c         running.
c
c         TTIME is an independent timer for taking a
c         temperature reading.
c         ---------------------------------------------
c
       if (dspini .ne. 2) then
          step = 0
          runtim = 100000
          go to 900
       endif
c
       ttime  = min (ttime + 1, 3000)
       runtim = runtim + 1
c
c
c         Here is a place we can do some
c         initializing.  It will only execute
c         the first time AFTER the first time
c         the DSP is initialized successfully.
c         ------------------------------------
c
       if     (step .eq. - 1) then
          pctvlf = pctval
          pctvls = pctval
c
c
c         Normal start of clock processing.
c         ---------------------------------
c
       elseif (step .eq. 0) then
c
c
c         Logic for tracking the percentile value of
c         the image.  NLO is the number of pixels in
c         the previous frame lower than the previous
c         estimate of PCTVAL.
c
c         If NLO is less than the percentile number
c         of pixels, PCTVAL needs to increase.  If
c         NLO is greater then the percentile number,
c         PCTVAL needs to increase.
c
c         The new estimate of PCTVAL is derived
c         according to one of 3 cases:
c
c         1.  The value of NLO crossed over the %tile
c         number between the previous frame and the
c         current frame.  Change the value of PCTVAL
c         1/2 as much as last time, in the opposite
c         direction.
c
c         2.  The value of NLO did not cross over the
c         %tile number:
c
c         2a:  The value of NLO crossed over the
c         previous time.  Change PCTVAL the same
c         amount as last time.
c
c         2b:  The value of NLO did not cross over
c         the previous time.  Double the change in
c         PCTVAL.
c
c         Also, save the previous value of PCTVAL, so
c         that it will be possible to form the boxcar
c         average in order to avoid ping-ponging by
c         one DN.
c         -------------------------------------------
c
          if ((pctset .gt. 0) .and. pctflg) then
             upflg = nlo .lt. ((pctset * imgsiz) / 100)
             cross = prflg .ne. upflg
             if (cross) then
                dval = dval / 2
                dval = max (dval, 1)
                prvx = .true.
             else
                if (prvx) then
                   prvx = .false.
                else
                   dval = 2 * dval
                   dval = min (dval, 1024)
                endif
             endif
c
             pctprv = pctval
             if (upflg) then
                pctval = pctval + dval
                pctval = min (pctval, 16384)
             else
                pctval = pctval - dval
                pctval = max (pctval, 0)
             endif
             prflg = upflg
c
             nlo = 0
             pctflg = .false.
c
c
c         Save pctval for full-frame
c         and subraster separately.
c         --------------------------
c
             if (imgsiz .lt. 2000) then
                pctvls = pctval
             else
                pctvlf = pctval
             endif
          endif
c
c
c         When the exposure time has
c         elapsed, initiate a CCD read.
c         -----------------------------
c
          if (runtim .lt. exptim) go to 900
c
c
c         First of all, scan the list of control values
c         and transmit any new entries.  There are 48
c         values, but entry "0" is the command flag and
c         is sent separately.  So entries 1... 47
c         correspond exactly to the numbering in the
c         DSP.  Even if they all change, there will be
c         36 microsec to send each one within the
c         interrupt time of 1.7165 millisec.
c
c         The command to load an item is x'FC03',
c         followed by the item number and the value.
c
c         The current in the thermoelectric cooler is a
c         special case.  Once the value is set, the DSP
c         must be commanded to use the new value.
c
c         The amplifier parameters (18, 19, and 20) are
c         another special case.  If any of the values
c         have been set, the DSP must be commanded to
c         use the the new values.
c         ---------------------------------------------
c
          ampfl = .false.
          dacfl = .false.
          do 200 i = 1, 47
             if (dspflg (i) .eq. 1) then
                itry = dspsend (x'FC03')
                i4   = i
                itry = dspsend (i2 (1))
                itry = dspsend (i2 (2))
                i4   = dspval (i)
                itry = dspsend (i2 (1))
                itry = dspsend (i2 (2))
c
c
c         Read back the value to make sure
c         everything is OK.  Return an error
c         value of 3 if the I/O is OK but
c         the value is wrong.
c         --------------------------------
c
                ier = dspget (i, i4)
                if (ier .eq. 0) then
                   if (i4 .eq. dspval (i)) then
                      dspflg (i) = 0
                      if     (i .eq. 18)    then
                         ampfl = .true.
                      elseif (i .eq. 19)    then
                         ampfl = .true.
                      elseif (i .eq. 20)    then
                         ampfl = .true.
                      elseif (i .eq. x'1A') then
                         itry = dspsend (x'F30C')
                      elseif (i .eq. 27)    then
                         dacfl = .true.
                      endif
c
c
c         Read-back error:
c            DSPINI = 5
c            DSPERR = item number
c         -----------------------
c
                   else
                      dspini = 5
                      dsperr = i
                      go to 900
                   endif
c
c
c         FIFO or time-out:
c            DSPINI = 3 or 4
c            DSPERR = item number
c         -----------------------
c
                else
                   dspini = ier + 2
                   dsperr = i
                   go to 900
                endif
             endif
  200     continue
c
c
c         Use new amplifier values.
c         -------------------------
c
          if (ampfl) then
             itry = dspsend (x'F20D')
          endif
c
c
c         Use new DAC values.
c         -------------------
c
          if (dacfl) then
             itry = dspsend (x'F50A')
          endif
c
c
c         Make certain that the FIFO is empty.
c         If the FIFO counter is zero, check
c         that the receive data ready bit is
c         also off.
c         ------------------------------------
c
          fifoct = fifchk ()
          if (fifoct .ne. 0) then
             dspini = 3
             dsperr = 101
             go to 900
          else
             mbox  = inpd (iobase + x'1C')
             if (and (mbox, x'02000000') .ne. 0) then
                dspini = 6
                dsperr = 102
                go to 900
             endif
          endif
c
c
c         Now, compute the number of words in
c         the image, and send the command to
c         initiate the read (x'F708').  Also,
c         latch the averaging parameter.
c
c         If the image size changes, don't
c         use the leaky-memory.
c         -----------------------------------
c
          imgprv = imgsiz
          imgsiz = dspval (2) * dspval (3)
          imgptr = 0
          imgflg = or (imgflg, x'FF00')
          itry = dspsend (x'F708')
          acqtim = runtim
          runtim = 0
          dtime = 0
          if (xfsflg .eq. 0) then
             avp  = avpf
             avp1 = avpf + 1
          else
             avp  = avps
             avp1 = avps + 1
          endif
          if (imgsiz .ne. imgprv) avp = 0
c
c
c         Restore PCTVAL separately for
c         full-frame and subraster.
c         -----------------------------
c
          if (imgsiz .lt. 2000) then
             pctval = pctvls
          else
             pctval = pctvlf
          endif
c
          step = 1
c
c
c         There won't be any data until the
c         frame transfer is complete, so now
c         would be a good time to do a little
c         bit of housekeeping.
c         -----------------------------------
c
       elseif (step .eq. 1) then
          dtime = dtime + 1
          step = 2
c
c
c         Read the data out of the FIFO.
c
c         First, test for a time-out.  In this step,
c         DTIME is the time that has elapsed since
c         the last data read.  The frame transfer
c         interval (0.2 sec, for now) must elapse
c         before the first data appears.  But after
c         that, data should appear much more
c         frequently.
c         ------------------------------------------
c
       elseif (step .eq. 2) then
          dtime = dtime + 1
          if ( ((imgptr .eq. 0) .and. (dtime .gt. 120)) .or.
     :         ((imgptr .gt. 0) .and. (dtime .gt. 12 ))      ) then
             dspini = 7
             dsperr = imgsiz - (imgptr + fifoct)
             step = 0
             go to 900
          endif
c
c
c         The buffer has 1024 words.  If there are
c         more than 1024 words left in the image,
c         then there will be another buffer-read
c         after this one.  If the number of words
c         left in the image is .le. 1024, then this
c         will be the last one.
c
c         It looks like there might not be enough
c         time for 1024 on some computers.  Try 512.
c
c         512 isn't enough to keep up with the read-
c         out in binned-2 mode.  Try 600.
c         ------------------------------------------
c
          fifoct = fifchk ()
          if ((imgsiz - imgptr) .gt. nbuf) then
             if (fifoct .ge. nbuf) then
                call pcirip (pcibuf, pt1map, nbuf)
                call hi2clr (pcibuf, nbuf)
                if (avp .eq. 0) then
                   do 300 i = 1, nbuf
                      imgptr = imgptr + 1
                      imgbuf (imgptr) = pcibuf (i)
                      if (imgbuf (imgptr) .lt. pctval) nlo = nlo + 1
  300              continue
                else
                   do 320 i = 1, nbuf
                      imgptr = imgptr + 1
                      imgbuf (imgptr) = (pcibuf (i) +
     :                                  (avp * imgbuf (imgptr)) )
     :                                  / avp1
                      if (imgbuf (imgptr) .lt. pctval) nlo = nlo + 1
  320              continue
                endif
                dtime = 0
             endif
c
          else
             if (fifoct .ge. (imgsiz - imgptr)) then
                ibuf = imgsiz - imgptr
                call pcirip (pcibuf, pt1map, ibuf)
                call hi2clr (pcibuf, ibuf)
                if (avp .eq. 0) then
                   do 400 i = 1, ibuf
                      imgptr = imgptr + 1
                      imgbuf (imgptr) = pcibuf (i)
                      if (imgbuf (imgptr) .lt. pctval) nlo = nlo + 1
  400              continue
                else
                   do 420 i = 1, ibuf
                      imgptr = imgptr + 1
                      imgbuf (imgptr) = (pcibuf (i) +
     :                                  (avp * imgbuf (imgptr)) )
     :                                  / avp1
                      if (imgbuf (imgptr) .lt. pctval) nlo = nlo + 1
  420              continue
                endif
                imgflg = x'00FF'
                pctflg = .true.
c
c
c         If there is time left in the
c         exposure, read the temperature.
c         -------------------------------
c
                if ((exptim - runtim) .gt. 24) then
                   step = 11
                else
                   step = 0
                endif
             endif
          endif
c
c
c         Steps 11 and 12 read the CCD
c         temperature.  The sensor is always
c         on anyway, so check the temperature
c         on every opportunity.  The old code
c         checked only once every 5 seconds.
c
c         Send the command to read the
c         temperature (x'F40B'), then wait 32
c         msec.
c         -----------------------------------
c
       elseif (step .eq. 11) then
ccc          if (ttime .lt. 2913) then
ccc             step = 0
ccc             go to 900
ccc          endif
          itry = dspsend (x'F40B')
          ttime = 0
          dtime = 0
          step  = 12
c
c
c         32 msec = 18.6 clock cycles.  DSPGET
c         will probably time out with a delay 18
c         cycles.  To take account of timing
c         jitter within the clock interrupt
c         (depending on what else is going on in
c         a given cycle), use a value of 20.
c         --------------------------------------
c
       elseif (step .eq. 12) then
          dtime = dtime + 1
          if (dtime .ge. 20) then
             ier = dspget (x'0011', i4)
             tmpval = i4
             tmperr = ier
             ier = dspget (x'0018', i4)
             vbsval = i4
             tmperr = tmperr + (10 * ier)
             step = 0
          endif
c
       endif
c
  900  continue
c
       return
       end
c
c
c         Get a 24-bit data value
c         from DSP memory. Error
c         status on return:
c            0... OK
c            1... FIFO not empty
c            2... DSP timeout
c         -----------------------
c
       integer*4 function dspget (item, value)
c
       implicit none
c
       integer*4 item, value, i4, itry
       integer*4 fifoct, ier, mpeek
       integer*4 itry, dspsend, fifchk
       integer*4 iobase, pt1map, pt2map
       integer*2 i2 (2)
c
       common / pcicom / iobase, pt1map, pt2map
c
       equivalence (i4, i2 (1))
c
c
c         The FIFO must be empty or
c         there is an error.
c         -------------------------
c
       fifoct = fifchk ()
       if (fifoct .ne. 0) then
          ier = 1
          go to 900
       endif
c
c
c         x'FD02' is the "GET" command
c         to the DSP, followed by the
c         24-bit item number.
c         ----------------------------
c
       itry = dspsend (x'FD02')
       i4   = item
       itry = dspsend (i2 (1))
       itry = dspsend (i2 (2))
c
c
c         When 2 words appear in the
c         FIFO, collect the reply.
c         --------------------------
c
       do 100 itry = 1, 100
          fifoct = fifchk ()
          if (fifoct .ge. 2) then
             i2 (1) = mpeek (pt1map)
             i2 (2) = mpeek (pt1map)
             value = i4
             ier = 0
             go to 900
          endif
  100  continue
c
c
c         DSP response time-out.
c         ----------------------
c
       ier = 2
c
  900  continue
       dspget = ier
       return
       end
c
c
c         Get the value in the
c         FIFO count register.
c         --------------------
c
       integer*4 function fifchk ()
c
       implicit none
c
       integer*4 fifoct, mpeek
       integer*4 iobase, pt1map, pt2map
       integer*4 fifmin, fifval, fifmax
c
       common / pcicom / iobase, pt1map, pt2map
       common / fifcom / fifmin, fifval, fifmax
c
       fifoct = mpeek (pt2map)
       if (fifmin .gt. fifoct) fifmin = fifoct
       if (fifmax .lt. fifoct) fifmax = fifoct
       fifval = fifoct
       fifchk = fifoct
c
       return
       end
