c
c
c         Load DSP program for Magellan guider camera.
c
c         The flag DSPINI has 3 possible values:
c            DSPINI = 0;  DSP initialization failure.
c            DSPINI = 1;  DSP initialization in progress.
c            DSPINI = 2;  DSP initialization success.
c         -----------------------------------------------
c
       subroutine dspload
c
       implicit none
c
       real*4    r4
       integer*4 i, config (6), i4, srec, nrec
       integer*4 dspini, dsperr, dspget
       integer*4 binf, bins
       integer*4 dspflg (47), dspval (47)
       integer*4 timer1, timer2, timer3, timer4
       integer*4 i24, n24, addrs, memsp, step
       integer*4 sec_100, t0, dt, ios, chksum
       integer*4 dspsend, mpeek, itry, xmit4 (50)
       integer*4 pcidev, iobase, pt1adr, pt2adr
       integer*4 mapdev, pt1map, pt2map, fifoct
       integer*4 eax, ebx, ecx, edx, esi, edi
       logical*4 opnflg, mapini
       integer*2 i2 (2), xmit2 (100)
       character line*80, msg*37
c
       common / pcicom / iobase, pt1map, pt2map
       common / dspcom / dspini, dsperr, dspflg, dspval
       common / timcom / timer1, timer2, timer3, timer4
       common / bincom / binf, bins
c
       equivalence (i4, i2 (1)), (xmit4 (1), xmit2 (1))
c
       data srec / 4 /, step / 1 /, opnflg / .false. /
       data mapini / .false. /
c
c
c         Check that PCI BIOS is present.
c         -------------------------------
c
       if    (step .eq. 1) then
          eax = x'B101'
          ebx = 0
          ecx = 0
          edx = 0
          esi = 0
          edi = 0
c
          call putmsg ('989; Checking for PCI BIOS--')
          call int026 (eax, ebx, ecx, edx, esi, edi)
c
          if ( (and (eax, x'FF00') .eq. 0) .and.
     :         (and (edx, x'FFFF') .eq. x'4350') ) then
             msg = '988; PCI BIOS vx.x detected'
             i4 = and (ebx, x'FF00') / 256
             msg (16:16) = char (i4 + 48)
             i4 = and (ebx, x'00F0') / 16
             msg (18:18) = char (i4 + 48)
             call putmsg (msg)
             step = 2
          else
             call putmsg ('001: PCI BIOS not present')
             step = 99
          endif
c
c
c         Find the interface card.
c         ------------------------
c
       elseif (step .eq. 2) then
          eax = x'B102'
          ebx = 0
          ecx = x'5920'
          edx = x'10E8'
          esi = 0
          edi = 0
c
          call putmsg ('987; Checking for PCI interface--')
          call int026 (eax, ebx, ecx, edx, esi, edi)
c
          if (and (eax, x'FF00') .eq. 0) then
             pcidev = and (ebx, x'00FF')
             msg = '986; PCI device number is xxxx'
             write (msg (27:30), '(z4.4)') pcidev
             call putmsg (msg)
             step = 3
          else
             call putmsg ('002: PCI interface not found')
             step = 99
          endif
c
c
c         Read configuration space for
c         the PCI interface.
c         ----------------------------
c
       elseif (step .eq. 3) then
          call putmsg ('985; Reading config registers--')
c
          do 100 i = 8, 13
             eax = x'B109'
             ebx = pcidev
             ecx = 0
             edx = 0
             esi = 0
             edi = 2 * i
c
             call int026 (eax, ebx, ecx, edx, esi, edi)
c
             config (i - 7) = ecx
  100     continue
c
c
c         Derive the I/O space base address,
c         and the physical memory addresses
c         of the two pass-through registers.
c         ----------------------------------
c
          i2 (1) = and (config (1), x'FFFE')
          i2 (2) = config (2)
          iobase = i4
c
          i2 (1) = config (3)
          i2 (2) = config (4)
          pt1adr = i4
c
          i2 (1) = config (5)
          i2 (2) = config (6)
          pt2adr = i4
c
          msg = '984; I/O base adr: xxxxxxxx'
          write (msg (20:27), '(z8.8)') iobase
          call putmsg (msg)
c
          msg = '983; PT1, PT2 adr: xxxxxxxx, xxxxxxxx'
          write (msg (20:27), '(z8.8)') pt1adr
          write (msg (30:37), '(z8.8)') pt2adr
          call putmsg (msg)
c
c
c         Map the pass-through registers into
c         program memory space.  Note that an
c         error will occur if the addresses
c         are not on a 4K page boundary.
c         -----------------------------------
c
          pt1map = mapdev (pt1adr, 4096)
          pt2map = mapdev (pt2adr, 4096)
          mapini = .true.
c
          msg = '982; PT1, PT2 map: xxxxxxxx, xxxxxxxx'
          write (msg (20:27), '(z8.8)') pt1map
          write (msg (30:37), '(z8.8)') pt2map
          call putmsg (msg)
c
          step = 4
c
c
c         Reset the DSP.  The reset control register is
c         at iobase + x'3C'.  The Add-On reset bit is
c         x'1000000'.  Toggle the bit high for 0.1 second,
c         then wait 1 second.  Note that 1 clock tick =
c         1.7165 msec; 1 sec = 582.6 clock ticks.
c         ------------------------------------------------
c
       elseif (step .eq. 4) then
          call putmsg ('981; Resetting DSP--')
          i4 = x'1000000'
          call outpd (iobase + x'3C', i4)
          t0 = timer1
          step = 5
c
       elseif (step .eq. 5) then
          dt = timer1 - t0
          if (dt .lt. 0) dt = dt + 1000000000
ccc          if (dt .gt. 582) then
          if (dt .gt. 58) then
             i4 = 0
             call outpd (iobase + x'3C', i4)
             t0 = timer1
             step = 6
          endif
c
c
c         When the wait interval is
c         finished, empty the FIFO.
c         -------------------------
c
       elseif (step .eq. 6) then
          dt = timer1 - t0
          if (dt .lt. 0) dt = dt + 1000000000
          if (dt .gt. 582) then
c
             fifoct = mpeek (pt2map)
             msg = '980; Initial FIFO count =xxxxx'
             call iform (fifoct, msg (26:30), 1)
             call putmsg (msg)
c
             if ( (fifoct .gt. 0)   .and.
ccc     :            (fifoct .lt. 1000)      ) then
     :            (fifoct .lt. 100000)      ) then
                do 250 i = 1, fifoct
                   i4 = mpeek (pt1map)
  250           continue
             endif
c
             fifoct = mpeek (pt2map)
             msg = '979;   Final FIFO count =xxxxx'
             call iform (fifoct, msg (26:30), 1)
             call putmsg (msg)
c
             step = 7
          endif
c
c
c         Open the S-record file.
c         -----------------------
c
       elseif (step .eq. 7) then
          call putmsg ('978; Loading DSP program--')
          open (unit = srec, file = 'ccd.s',
     :          status = 'old', iostat = ios)
          if (ios .ne. 0) then
             call putmsg ('003: Error opening S-record file')
             opnflg = .false.
             step = 99
          else
             opnflg = .true.
             nrec = 0
             step = 8
          endif
c
c
c         Read the S-record into the binary
c         data array.
c         ---------------------------------
c
       elseif (step .eq. 8) then
          read (srec, '(a80)', iostat = ios) line
          if     (ios .eq. -1) then
             call putmsg ('004: EOF encountered in S-record file')
             msg =        '005: at record xxxx'
             call iform (nrec, msg (16:19), 1)
             call putmsg (msg)
             step = 99
c
          elseif (line (1:1) .eq. 'S') then
             nrec = nrec + 1
c
             read (line (3:4), '(z2)') i
             n24 = (i - 3) / 3
             i = 5
c
             if     (line (2:2) .eq. '0') then
                read (line (i:), '(z4)') addrs
                memsp = and (addrs, x'FF')
                i = i + 4
c
             elseif (line (2:2) .eq. '2') then
                read (line (i:), '(z6)') addrs
                i = i + 6
c
             elseif (line (2:2) .eq. '8') then
                read (line (i:), '(z6)') addrs
                memsp = 8
                i = i + 6
c
             else
                call putmsg ('006: Unrecognized S-record format')
                msg =        '007: at record xxxx'
                call iform (nrec, msg (16:19), 1)
                step = 99
             endif
c
c
c         Continue reading the binary data,
c         except for "S0" records which are
c         not transmitted.
c         ---------------------------------
c
             if ( (line (2:2) .eq. '2') .or.
     :            (line (2:2) .eq. '8')      ) then
                xmit2 (1) = n24
                xmit2 (2) = memsp
                xmit4 (2) = addrs
                i24 = 0
c
  350           continue
                if (i24 .lt. n24) then
                   i24 = i24 + 1
                   read (line (i:), '(z6)') xmit4 (i24 + 2)
                   i = i + 6
                   go to 350
                endif
c
c
c         Transmit the binary data array
c         and calculate the checksum.
c         ------------------------------
c
                chksum = 0
                do 400 i = 1, (2 * (n24 + 2))
                   itry = dspsend (xmit2 (i))
                   chksum = chksum + xmit2 (i)
  400           continue
                chksum = and (chksum, x'FFFF')
c
c
c         Look for the checksum in the reply.
c         -----------------------------------
c
                if (line (2:2) .ne. '8') then
                   i = 1
  450              continue
                   i = i + 1
                   fifoct = mpeek (pt2map)
                   if     (fifoct .eq. 1) then
                      i4 = mpeek (pt1map)
                      if ((i4 - chksum) .ne. 0) then
                         msg = '008: Checksum error at record xxxx'
                         call iform (nrec, msg (31:34), 1)
                         call putmsg (msg)
                         msg = '009: expected: xxxx,  received: xxxx'
                         write (msg (16:19), '(z4.4)') chksum
                         write (msg (33:36), '(z4.4)') i4
                         call putmsg (msg)
                         step = 99
                      endif
c
                   elseif (fifoct .gt. 1) then
                      write (*, '(a)')
                      call putmsg
     :                   ('010: FIFO error during program load')
                      msg = '011: record = xxxx, FIFO count = xxxx'
                      call iform (nrec,   msg (15:18), 1)
                      call iform (fifoct, msg (34:37), 1)
                      call putmsg (msg)
                      step = 99
c
                   elseif (i .lt. 1000) then
                      go to 450
c
                   else
                      msg = '012: Checksum timeout at record xxxx'
                      call iform (nrec, msg (33:36), 1)
                      call putmsg (msg)
                      step = 99
                   endif
c
c
c         Except for "S8" records --
c         wait one second.
c         --------------------------
c
                else
                   t0 = timer1
                   step = 9
                endif
c
             endif
c
          endif
c
       elseif (step .eq. 9) then
          dt = timer1 - t0
          if (dt .lt. 0) dt = dt + 1000000000
          if (dt .gt. 582) then
             msg = '977; xxxx S-records read, no errors'
             call iform (nrec, msg (6:9), 1)
             call putmsg (msg)
             step = 10
          endif
c
c
c         Test the DSP program by reading the
c         TEC current setting, which we need
c         to know anyway.
c         -----------------------------------
c
       elseif (step .eq. 10) then
          call putmsg ('976; Running DSP function test--')
          itry = dspget (x'001A', i4)
          if     (itry .eq. 0) then
             fifoct = mpeek (pt2map)
             if (fifoct .eq. 0) then
                i4 = i4 - 4096
                r4 = 5.e-3 * i4
                call vbox (1, r4)
                call putmsg ('975; DSP function test OK')
                step = 98
             else
                msg = ('013: Error, final FIFO count = xxxx')
                call iform (fifoct, msg (32:35), 1)
                call putmsg (msg)
                step = 99
             endif
          elseif (itry .eq. 1) then
             call putmsg ('014: DSP test FIFO count error')
             step = 99
          elseif (itry .eq. 2) then
             call putmsg ('015: DSP test timeout error')
             step = 99
          endif
c
c
c         DSP initialization complete:
c         set the default values for
c         the DSP control parameters.
c         ----------------------------
c
       elseif (step .eq. 98) then
          close (srec)
          call dspset (binf)
          call putmsg ('980; DSP initialization success')
          dspini = 2
          step   = 4

       elseif (step .eq. 99) then
          if (opnflg) close (srec)
          call putmsg ('020: DSP initialization failure')
          dspini = 0
          if (mapini) then
             step = 4
          else
             step = 1
          endif
c
       endif
c
       return
       end
c
c
c         Send a 16-bit word to the DSP by writing
c         to pass-thru register 1.  Wait for the
c         parallel-to-serial conversion to finish
c         before returning to the calling program.
c         ----------------------------------------
c
       integer*4 function dspsend (val)
c
       implicit none
c
       integer*4 val, mbox, itry, inpd
       integer*4 iobase, pt1map, pt2map
       logical*4 busy, bflg
c
       common / pcicom / iobase, pt1map, pt2map
c
c
c         Put the value into the transmit register.
c         -----------------------------------------
c
       call mpoke (pt1map, val)
c
c
c         The mailbox register is located at offset x'1C'.
c         The transmit register is busy while mailbox bit
c         x'01000000' is true.
c
c         However, the busy bit doesn't necessarily come
c         on immediately.  So the transmit operation is
c         not complete until the busy bit first goes high,
c         then goes low.  The typical number of tries for
c         the transmit register to clear is 3 or 4.  The
c         number of tries is returned as the value of
c         dspsend.
c
c         If an interrupt occurs at just the wrong time,
c         then it might be possible to miss the busy bit
c         completely.  So the number of tries is limited
c         to a maximum value of 10.
c         ------------------------------------------------
c
       itry = 1
       bflg = .false.
c
  100  continue
       mbox = inpd (iobase + x'1C')
       busy = (and (mbox, x'01000000') .ne. 0)
       if (busy) bflg = .true.
c
       if ( (busy .or. (.not. bflg)) .and.
     :      (itry .lt. 10          )       ) then
          itry = itry + 1
          go to 100
       endif
c
       dspsend = itry
       return
       end
c
c
c         For an integer*4 array, set the
c         two most significant bytes of each
c         entry to zero.
c         ----------------------------------
c
       subroutine hi2clr (i4vals, n)
c
       implicit none
c
       integer*4 n, n2, i
       integer*2 i4vals (1)
c
       n2 = 2 * n
c
       do 100 i = 2, n2, 2
          i4vals (i) = 0
  100  continue
c
       return
       end
c
c
c         Setup to transmit default
c         control values to the DSP.
c         --------------------------
c
       subroutine dspset (fnc)
c
       implicit  none
c
       real*4    r4
       integer*4 fnc, dspini, dsperr, i4
       integer*4 dspflg (47), dspval (47)
       integer*4 incur, xcur (5), ycur (5)
       integer*4 xfsflg, exftim, exstim
       integer*4 exptim, runtim, acqtim
       integer*4 binf, bins
       integer*4 gain, usec, hdir, tec, gate
c
       common / dspcom / dspini, dsperr, dspflg, dspval
       common / xfscom / xfsflg, exftim, exstim
       common / expcom / exptim, runtim, acqtim
       common / bincom / binf, bins
       common / cr1com / incur, xcur, ycur
       common / ampcom / gain, usec, hdir, tec, gate
c
c
c         We need to set a semaphore for the
c         clock routine so it will not send
c         a partially complete set-up to the
c         DSP.  Setting EXPTIM to a big
c         number will do just fine.
c         ----------------------------------
c
       exptim = 1000000
c
c
c         The setup is controlled by the
c         value of fnc:
c            2:  full frame, bin 2
c            4:  full frame, bin 4
c
c           -2:  subraster,  bin 2
c           -4:  subraster,  bin 4
c         ------------------------------
c
c
c         Full frame, bin 2:
c         ------------------
c
       if     (fnc .eq. 2) then
          xfsflg = 0
c
c
c         Number of rows.  To eliminate the
c         dark line at the start of the frame,
c         read out 501 lines and throw away
c         the first one.
c         ------------------------------------
c
          dspflg (2) = 1
          dspval (2) = 501
c
c
c         Number of columns.
c         ------------------
c
          dspflg (3) = 1
          dspval (3) = 500
c
c
c         Vertical binning.
c         -----------------
c
          dspflg (8) = 1
          dspval (8) = 2
c
c
c         Horizontal binning.
c         -------------------
c
          dspflg (9) = 1
          dspval (9) = 2
c
c
c         Vertical skip.  The value for
c         binned-4 mode is 3.  But throwing
c         away the first line in each case
c         means we need to skip 5 here.
c
c         Changing the vertical skip
c         doesn't eliminate the dark line
c         at the start of the frame.
c         ---------------------------------
c
          dspflg (10) = 1
          dspval (10) = 5
c
c
c         Horizontal skip.  The default
c         value is 18.
c         -----------------------------
c
          dspflg (11) = 1
          dspval (11) = 18
c
c
c         Full frame, bin 4:
c         ------------------
c
       elseif (fnc .eq. 4) then
          xfsflg = 0
c
c
c         Number of rows.  To eliminate the
c         dark line at the start of the frame,
c         read out 251 lines and throw away
c         the first one.
c         ------------------------------------
c
          dspflg (2) = 1
          dspval (2) = 251
c
c
c         Number of columns.
c         ------------------
c
          dspflg (3) = 1
          dspval (3) = 250
c
c
c         Vertical binning.
c         -----------------
c
          dspflg (8) = 1
          dspval (8) = 4
c
c
c         Horizontal binning.
c         -------------------
c
          dspflg (9) = 1
          dspval (9) = 4
c
c
c         Vertical skip.  The default value
c         is 3.  Use the default value for
c         the binned-4 mode, and adjust the
c         other modes accordingly.
c         ---------------------------------
c
          dspflg (10) = 1
          dspval (10) = 3
c
c
c         Horizontal skip.  The default
c         value is 18.
c         -----------------------------
c
          dspflg (11) = 1
          dspval (11) = 18
c
c
c         Subraster, bin 2:
c         -----------------
c
       elseif (fnc .eq. -2) then
          xfsflg = 1
c
c
c         Number of rows.  To eliminate the
c         dark line at the start of the frame,
c         read out 33 lines and throw away
c         the first one.
c         ------------------------------------
c
          dspflg (2) = 1
          dspval (2) = 37
c
c
c         Number of columns.
c         ------------------
c
          dspflg (3) = 1
          dspval (3) = 36
c
c
c         Vertical binning.
c         -----------------
c
          dspflg (8) = 1
          dspval (8) = 2
c
c
c         Horizontal binning.
c         -------------------
c
          dspflg (9) = 1
          dspval (9) = 2
c
c
c         Vertical skip.  The value for binned-4
c         mode is 3.  But throwing away the first
c         line in each case means we need to skip
c         5 here.
c
c         Skip only two lines at a time, in order
c         to keep the registration of the binned
c         pixels on the array constant.
c         ---------------------------------------
c
          i4 = (ycur (5) / 10) - 35
          i4 = i4 / 2
          dspflg (10) = 1
          dspval (10) = 5 + (2 * i4)
c
c
c         Horizontal skip.  The default value is
c         18.  Skip only two pixels at a time, in
c         order to keep the pixel registraion
c         constant.
c         ---------------------------------------
c
          i4 = (xcur (5) / 10) - 35
          i4 = i4 / 2
          dspflg (11) = 1
          dspval (11) = 18 + (2 * i4)
c
c
c         Subraster, bin 4:
c         -----------------
c
       elseif (fnc .eq. -4) then
          xfsflg = 1
c
c
c         Number of rows.  To eliminate the
c         dark line at the start of the frame,
c         read out 33 lines and throw away
c         the first one.
c         ------------------------------------
c
          dspflg (2) = 1
          dspval (2) = 20
c
c
c         Number of columns.
c         ------------------
c
          dspflg (3) = 1
          dspval (3) = 19
c
c
c         Vertical binning.
c         -----------------
c
          dspflg (8) = 1
          dspval (8) = 4
c
c
c         Horizontal binning.
c         -------------------
c
          dspflg (9) = 1
          dspval (9) = 4
c
c
c         Vertical skip.  The default value for
c         binned-4 mode is 3.  Skip only four
c         lines at a time, in order to keep the
c         registration of the binned pixels on
c         the array constant.
c         -------------------------------------
c
          i4 = (ycur (5) / 10) - 35
          i4 = i4 / 4
          dspflg (10) = 1
          dspval (10) = 3 + (4 * i4)
c
c
c         Horizontal skip.  The default value
c         is 18.
c         -----------------------------------
c
          i4 = (xcur (5) / 10) - 35
          i4 = i4 / 4
          dspflg (11) = 1
          dspval (11) = 18 + (4 * i4)
c
       endif
c
c
c         Amplifier parameters.
c         ---------------------
c
       dspflg (18) = 1
       dspval (18) = gain
c
       dspflg (19) = 1
       dspval (19) = usec
c
       dspflg (20) = 1
       dspval (20) = hdir
c
       dspflg (27) = 1
       dspval (27) = gate
c
c
c         Thermoelectric cooler current.
c         ------------------------------
c
       dspflg (x'1A') = 1
       dspval (x'1A') = tec
       i4 = tec - 4096
       r4 = 5.e-3 * i4
       call vbox (1, r4)
c
c
c         Set the exposure time last of
c         all, full-frame or subraster.
c         This alerts the clock routine
c         that it is OK to proceed.
c         -----------------------------
c
       if (xfsflg .eq. 0) then
          exptim = exftim
       else
          exptim = exstim
       endif
c
       return
       end
