c
c
c         Format incoming images from the Magellan guider
c         cameras and send them to the screen.
c
c         The screen image is 250 x 250.  But the screen
c         buffer is 252 x 250, so the horizontal size is
c         divisible by 4.  The lines start on a 4-byte
c         boundary.  By leaving the last two bytes in
c         each line with the screen background color, it
c         is possible to use BLKBMF4 to paint the screen,
c         which is faster.
c         -----------------------------------------------
c
       subroutine screen
c
       implicit  none
c
       real*4    rbuf (6000), zbuf (6000), ebuf (6000)
       real*4    r4, fit (6), sens, pa, px, tcref
       real*4    xerr, yerr, azerr, elerr, azerp, elerp, parall
       real*4    azg, elg, radpa, sinpa, cospa
       integer*4 eldx, eldy, azdx, azdy
       integer*4 dcdx, dcdy, radx, rady
       integer*4 elazx, elazy, dcrax, dcray
       integer*4 blk,gry,hlt,wht,red,yel,grn,bkg
       integer*4 gdrfl, xflip, yflip, box, avpf, avps
       integer*4 timer1, timer2, timer3, timer4
       integer*4 step, i4, iptr, ibuf, it, gbox
       integer*4 zero, zzero, zspan, bkgset
       integer*4 scrptr, nxtptr, zzz, ilo, ihi
       integer*4 xmod, ymod, ix, iy, savptr, yptr
       integer*4 sndtim, stime, dtime, xbuf (6000)
       integer*4 pctset, pctval, pctprv, sndptr
       integer*4 incur, xcur (5), ycur (5), imgtyp
       integer*4 imgsiz, imgptr, imgflg, pgptr
       integer*4 imgbuf (300000), scrbuf4 (15750)
       integer*4 sendct, sendsz, sndbuf (250000)
       integer*4 skyval, skyptr, skybuf (300000)
       logical*4 sendfl, gflag, skyflg, subflg
       logical*4 movbox
       integer*2 i2 (2)
       integer*1 scrbuf (63000), magbuf (6000)
       character msg*37
c
       common / timcom / timer1, timer2, timer3, timer4
       common / imgcom / imgsiz, imgptr, imgflg, imgbuf
       common / skycom / skyflg, subflg, skyval, skybuf
       common / cr1com / incur, xcur, ycur
       common / zptcom / zero, zspan
       common / zrocom / zzero, bkgset
       common / scbcom / scrbuf4
       common / typcom / imgtyp
       common / magcom / magbuf
       common / pctcom / pctset, pctval, pctprv
       common / sndcom / sendfl, sendct, sendsz, sndbuf
       common / sdtcom / sndtim, stime
       common / gdrcom / gdrfl, sens, xflip, yflip, pa, px, box
       common / gdscom / gflag, azg, elg, azerr, elerr, parall
       common / avpcom / avpf, avps
       common / colors / blk,gry,hlt,wht,red,yel,grn,bkg
       common / movcom / movbox

c
       equivalence (scrbuf4 (1), scrbuf (1)), (i4, i2)
c
       data  step / 0 /, azerp / 0. /, elerp / 0. /, it / 0 /
       data  sndptr / 0 /, skyptr / 0 /
       data  skyflg / .false. /, subflg / .false. /
       data  elazx /416/, elazy /62/, dcrax /456/, dcray /62/
c
c
c         Check the send-timer for the TCP link.
c         --------------------------------------
c
       if ((sndtim .ne. 0) .and. (.not. sendfl)) then
          dtime = timer1 - stime
          if (dtime .lt. 0) dtime = dtime + 1000000000
          dtime = (17165 * dtime) / 10000000
          call vbox (60, dtime)
          if (dtime .ge. sndtim) then
             sendfl = .true.
             stime = timer1
          endif
       else
          stime = timer1
       endif
c
c
c
c         The idea is to fill up the screen buffer
c         while the CCD is being read out, and then
c         to send the whole buffer to the screen as
c         soon as possible after the frame is
c         completed.
c
c         While the readout is in progress, imgptr
c         is increasing from zero.  The x'0200' bit
c         of imgflg is set when the readout begins,
c         and is re-set by the clock routine when
c         the readout is complete.
c         -----------------------------------------
c
c         The size of the image is computed by
c         the clock routine at the start of each
c         readout, so this is a good way to
c         distinguish the different readout modes.
c
c         The problem is that when a new frame
c         starts in a different mode, we have to
c         finish processing the last frame in the
c         old mode.  So latch the readout mode at
c         the beginning.
c         ---------------------------------------
c
       if (step .eq. 0) then
          if (and (imgflg, x'0200') .ne. 0) then
             scrptr = 0
c
c
c         Set the zero point so pctval corresponds
c         to the background screen setting.  If
c         PCTVAL and PCTPRV are close, use the
c         boxcar average to prevent ping-ponging
c         the intensity.
c         ----------------------------------------
c
             if (pctset .gt. 0) then
                if (abs (pctval - pctprv) .lt. 2) then
                   zzero = ((32 * (pctval + pctprv)) / bkgset) - zspan
                   skyval = (pctval + pctprv) / 2
                else
                   zzero = ((64 * (pctval)         ) / bkgset) - zspan
                   skyval = pctval
                endif
                zzero = (bkgset * zzero) / 64
             else
                zzero = zero
             endif
c
c
c         Subraster, binned 4.  There are 35 pixels
c         before the center point.  YMOD and XMOD
c         are used to determine whether the first
c         line and the first pixel in each line will
c         be used once, twice, three times or 4 times.
c         --------------------------------------------
c
             if (imgsiz .lt. 500) then
                imgtyp = - 4
                nxtptr = 38
                ymod = (ycur (5) / 10) - 35
                ymod = mod (ymod, 4)
c
c
c         Subraster, binned 2.  There are 35 pixels
c         before the center point.  YMOD and XMOD
c         are used to determine whether the first
c         line and the first pixel in each line will
c         be used once or twice.
c         ------------------------------------------
c
             elseif (imgsiz .lt. 2000) then
                imgtyp = - 2
                nxtptr = 72
                ymod = (ycur (5) / 10) - 35
                ymod = mod (ymod, 2)
c
             elseif (imgsiz .lt. 80000) then
                imgtyp = 4
                nxtptr = 500
c
             else
                imgtyp = 2
                nxtptr = 1500
             endif
c
c
c         If SENDFL is true, prepare a frame
c         to transmit via TCP/IP socket.
c
c         Note that SENDFL could go true in the
c         middle of a frame, so subsequent tests
c         should rely on SNDPTR.
c         --------------------------------------
c
             if     (sendfl) then
                sendsz = imgsiz
                sndptr = 1
             endif
c
c
c         If SKYFLG is true, store this frame
c         in the sky-subtraction buffer.
c
c         Note that SKYFLG could go true in the
c         middle of a frame, so subsequent tests
c         should rely on SKYPTR.
c         --------------------------------------
c
             if     (skyflg) then
                skyptr = 1
                skyflg = .false.
             endif
c
             step = 1
          endif
       endif
c
c
c         The next row of the screen is available when
c         imgptr is .ge. nxtptr.
c
c         But if the camera cycles immediately you could
c         miss the last few values of imgptr.
c
c         The x'0002' bit of imgflg is set when the
c         readout is complete, and re-set by this program
c         when the image has been sent to the screen.
c
c         If you miss the last few values of imgptr, the
c         x'0002' bit will indicate that the data is
c         available instead.
c
c         If SUBFLG is true, subtract the image in the
c         sky-subtraction buffer, but add the %tile value
c         back in so the auto-scaling will be OK.
c         -----------------------------------------------
c
       if (step .eq. 1) then
  100     continue
          if ( (imgptr                .ge. nxtptr) .or.
     :         (and (imgflg, x'0002') .ne.      0)      ) then
c
c
c         Full-frame, bin 2:
c         ------------------
c
             if     (imgtyp .eq. 2) then
                do 160 iptr = (nxtptr - 499), nxtptr, 2
c
                   i4 = imgbuf (iptr - 501) + imgbuf (iptr - 500)
     :                + imgbuf (iptr -   1) + imgbuf (iptr      )
c
                   if (subflg) then
                      i4 = i4 + skyval
     :                   - skybuf (iptr - 501) - skybuf (iptr - 500)
     :                   - skybuf (iptr -   1) - skybuf (iptr      )
                   endif
c
                   i4 = (64 * ((i4 / 4) - zzero)) / zspan
                   i4 = max (i4,  0)
                   i4 = min (i4, 63)
                   scrptr = scrptr + 1
                   scrbuf (scrptr) = i4 + 16
c
                   if (sndptr .gt. 0) then
                      sndbuf (sndptr      ) = imgbuf (iptr - 501)
                      sndbuf (sndptr +   1) = imgbuf (iptr - 500)
                      sndbuf (sndptr + 500) = imgbuf (iptr -   1)
                      sndbuf (sndptr + 501) = imgbuf (iptr      )
                      sndptr = sndptr + 2
                   endif
c
                   if (skyptr .gt. 0) then
                      skybuf (iptr - 501) = imgbuf (iptr - 501)
                      skybuf (iptr - 500) = imgbuf (iptr - 500)
                      skybuf (iptr -   1) = imgbuf (iptr -   1)
                      skybuf (iptr      ) = imgbuf (iptr      )
                   endif
  160           continue
                scrptr = scrptr + 2
c
                if (sndptr .gt. 0) sndptr = sndptr + 500
                if (scrptr .lt. 63000) then
                   nxtptr = nxtptr + 1000
                   go to 100
                else
                   step = 2
                endif
c
c
c         Full-frame, bin 4:
c         ------------------
c
             elseif (imgtyp .eq. 4) then
                do 180 iptr = (nxtptr - 249), nxtptr
                   i4 = imgbuf (iptr)
                   if (subflg) i4 = i4 + skyval - skybuf (iptr)
                   i4 = (64 * (i4 - zzero)) / zspan
                   i4 = max (i4,  0)
                   i4 = min (i4, 63)
                   scrptr = scrptr + 1
                   scrbuf (scrptr) = i4 + 16
c
                   if (sndptr .gt. 0) then
                      sndbuf (sndptr) = imgbuf (iptr)
                      sndptr = sndptr + 1
                   endif
c
                   if (skyptr .gt. 0) then
                      skybuf (iptr) = imgbuf (iptr)
                   endif
  180           continue
                scrptr = scrptr + 2
c
                if (scrptr .lt. 63000) then
                   nxtptr = nxtptr + 250
                   go to 100
                else
                   step = 2
                endif
c
c
c         Subraster, bin 2:  The subraster buffer
c         is organized as a 75 x 71 array so you
c         don't write over the beginning of the
c         next line with the end of the current
c         line.  Also store a real*4 version of
c         the subraster to use for centroiding.
c         ---------------------------------------
c
             elseif (imgtyp .eq. -2) then
                xmod = (xcur (5) / 10) - 35
                xmod = mod (xmod, 2)
                savptr = scrptr
                do 260 iptr = (nxtptr - 35), nxtptr
                   i4 = imgbuf (iptr)
                   r4 = i4
                   i4 = (64 * (i4 - zzero)) / zspan
                   i4 = max (i4, 0)
                   i4 = min (i4, 63)
                   scrptr = scrptr + 1
                   magbuf (scrptr) = i4 + 16
                   rbuf   (scrptr) = r4
                   if (ymod .eq. 0) then
                      magbuf (scrptr + 75) = i4 + 16
                      rbuf   (scrptr + 75) = r4
                   endif
                   if (xmod .eq. 0) then
                      scrptr = scrptr + 1
                      magbuf (scrptr) = i4 + 16
                      rbuf   (scrptr) = r4
                      if (ymod .eq. 0) then
                         magbuf (scrptr + 75) = i4 + 16
                         rbuf   (scrptr + 75) = r4
                      endif
                   else
                      xmod = 0
                   endif
c
                   if (sndptr .gt. 0) then
                      sndbuf (sndptr) = imgbuf (iptr)
                      sndptr = sndptr + 1
                   endif
  260           continue
c
                if (ymod .eq. 0) then
                   scrptr = savptr + 150
                else
                   scrptr = savptr +  75
                   ymod = 0
                endif
c
c
c         Note: 5325 = 75 * 71.
c         ---------------------
c
                if (scrptr .lt. 5325) then
                   nxtptr = nxtptr + 36
                   go to 100
                else
                   step = 2
                endif
c
c
c         Subraster, bin 4:
c         -----------------
c
             elseif (imgtyp .eq. -4) then
                xmod = (xcur (5) / 10) - 35
                xmod = mod (xmod, 4)
                savptr = scrptr
                do 280 iptr = (nxtptr - 18), nxtptr
                   i4 = (64 * (imgbuf (iptr) - zzero)) / zspan
                   i4 = max (i4, 0)
                   i4 = min (i4, 63)
c
                   scrptr = scrptr + 1
                   magbuf (scrptr) = i4 + 16
                   if (ymod .lt. 3) magbuf (scrptr +  75) = i4 + 16
                   if (ymod .lt. 2) magbuf (scrptr + 150) = i4 + 16
                   if (ymod .lt. 1) magbuf (scrptr + 225) = i4 + 16
c
                   if (xmod .lt. 3) then
                      scrptr = scrptr + 1
                      magbuf (scrptr) = i4 + 16
                      if (ymod .lt. 3) magbuf (scrptr +  75) = i4 + 16
                      if (ymod .lt. 2) magbuf (scrptr + 150) = i4 + 16
                      if (ymod .lt. 1) magbuf (scrptr + 225) = i4 + 16
                   endif
c
                   if (xmod .lt. 2) then
                      scrptr = scrptr + 1
                      magbuf (scrptr) = i4 + 16
                      if (ymod .lt. 3) magbuf (scrptr +  75) = i4 + 16
                      if (ymod .lt. 2) magbuf (scrptr + 150) = i4 + 16
                      if (ymod .lt. 1) magbuf (scrptr + 225) = i4 + 16
                   endif
c
                   if (xmod .lt. 1) then
                      scrptr = scrptr + 1
                      magbuf (scrptr) = i4 + 16
                      if (ymod .lt. 3) magbuf (scrptr +  75) = i4 + 16
                      if (ymod .lt. 2) magbuf (scrptr + 150) = i4 + 16
                      if (ymod .lt. 1) magbuf (scrptr + 225) = i4 + 16
                   endif
c
                   xmod = 0
c
                   if (sndptr .gt. 0) then
                      sndbuf (sndptr) = imgbuf (iptr)
                      sndptr = sndptr + 1
                   endif
  280           continue
c
                if     (ymod .eq. 0) then
                   scrptr = savptr + 300
                elseif (ymod .eq. 1) then
                   scrptr = savptr + 225
                elseif (ymod .eq. 2) then
                   scrptr = savptr + 150
                else
                   scrptr = savptr +  75
                endif
                ymod = 0
c
                if (scrptr .lt. 5325) then
                   nxtptr = nxtptr + 19
                   go to 100
                else
                   step = 2
                endif
c
             endif
c
          endif
c
c
c         The image is complete.  Put in the cursors.
c         This really needs to be in the same step as
c         writing to the screen and taking the cursors
c         out again.
c         --------------------------------------------
c
       elseif (step .eq. 2) then
          if (imgtyp .gt. 0) then
             call curwrt (scrbuf)
c
c
c         Send to screen.  Use opqwrt to set
c         the page address and for lines on
c         page boundaries.  Otherwise use
c         blkbmf2.
c
c         Page boundaries occur on screen line:
c            102: (1 * 65536) - (102 * 640) = 256
c            204: (2 * 65536) - (204 * 640) = 512
c            307: (3 * 65536) - (307 * 640) = 128
c            409: (4 * 65536) - (409 * 640) = 384
c         Note that we are not writing across the
c         boundary on line 204 (it's too close to
c         the end) so use 205 instead.
c            656224 = (10 * 65536) + 640 + 224
c         ---------------------------------------
c
             scrptr = 1
c
             call opqwrt (scrbuf4 (scrptr), 250, 224, 98)
             scrptr = scrptr + 63
c
             pgptr = 656224 + (98 * 640)
             do 310 iptr = 2, 4
                call blkbmf4 (scrbuf4 (scrptr), x'34', pgptr, 63)
                scrptr = scrptr + 63
                pgptr  = pgptr  + 640
  310        continue
c
             call opqwrt (scrbuf4 (scrptr), 250, 224, 102)
             scrptr = scrptr + 63
c
             pgptr = 656224 - 256
             do 320 iptr = 6, 107
                call blkbmf4 (scrbuf4 (scrptr), x'34', pgptr, 63)
                scrptr = scrptr + 63
                pgptr  = pgptr  + 640
  320        continue
c
             call opqwrt (scrbuf4 (scrptr), 250, 224, 205)
             scrptr = scrptr + 63
c
             pgptr = 656224 + 640 - 512
             do 330 iptr = 109, 209
               call blkbmf4 (scrbuf4 (scrptr), x'34', pgptr, 63)
               scrptr = scrptr + 63
               pgptr  = pgptr  + 640
  330        continue
c
             call opqwrt (scrbuf4 (scrptr), 250, 224, 307)
             scrptr = scrptr + 63
c
             pgptr = 656224 - 128
             do 340 iptr = 211, 250
                call blkbmf4 (scrbuf4 (scrptr), x'34', pgptr, 63)
                scrptr = scrptr + 63
                pgptr  = pgptr  + 640
  340        continue
c
c
c         Take the cursors out.
c         ---------------------
c
             call currst (scrbuf)
c
             step = 3
c
c
c         Once the subraster image is complete,
c         the very first thing to do is to guide.
c         During centroiding, reference all coords
c         to the subraster.  Then correct for the
c         position of the subraster at the end.
c
c         Assemble the subraster.  First the case
c         for binned-2.  Extract every-other pixel
c         and every-other row of the subraster.
c         ----------------------------------------
c
          else
             if (imgtyp .eq. - 2) then
                i4 = box / 2
                ilo = 36 - i4
                ihi = 36 + i4
                yptr = 75 * (ilo - 1)
c
c
c         I thought the coordinate values for each
c         point should be ix - 36 and iy - 36.
c         But the image ends up centered in the
c         guide box for ix - 35 and iy - 35 instead,
c         so what the heck.
c         ------------------------------------------
c
                ibuf = 0
                do 440 iy = ilo, ihi, 2
                   do 420 ix = ilo, ihi, 2
                      ibuf = ibuf + 1
                      i2 (1) = ix - 35
                      i2 (2) = iy - 35
                      xbuf (ibuf) = i4
                      zbuf (ibuf) = rbuf (yptr + ix)
                      ebuf (ibuf) = 1.
  420              continue
                   yptr = yptr + 150
  440           continue
c
                if (gdrfl .lt. 0) it = 0
                if (it .lt. 0) it = 0
                if (it .gt. 7) it = 0
                call plsphot (xbuf, zbuf, ebuf, ibuf, fit, it)
c
c
c         Depending on the box size, you may
c         have picked up all the right-hand
c         sides of the double pixels, or all
c         the left-hand sides.  A different
c         fudge is required in each case.
c         ----------------------------------
c
                if (mod (box / 2, 2) .eq. 0) then
                   xerr = fit (2) - 0.1 * mod (xcur (5)     , 20)
                   yerr = fit (3) - 0.1 * mod (ycur (5)     , 20)
                else
                   xerr = fit (2) - 0.1 * mod (xcur (5) - 10, 20)
                   yerr = fit (3) - 0.1 * mod (ycur (5) - 10, 20)
                endif
c
c
c         Rotate the corrections into AZ
c         and El coordinates.  See TCS
c         mouse routine for original code.
c         --------------------------------
c
                if (movbox) then
                    movbox = .false.
                    call curupr (5, nint(xerr*10.0), nint(yerr*10.0))
                else
                    radpa = abs (pa) / 57.29578
                    sinpa = sin (radpa)
                    cospa = cos (radpa)
                    if (pa .gt. 0.) then
                       azerr = + (xerr * cospa) - (yerr * sinpa)
                       elerr = - (xerr * sinpa) - (yerr * cospa)
                    else
                       azerr = - (xerr * cospa) - (yerr * sinpa)
                       elerr = + (xerr * sinpa) - (yerr * cospa)
                    endif
c
c
c         Scale guide corrections
c         to arcseconds.
c         -----------------------
c
                    azerr = azerr * px
                    elerr = elerr * px
c
c
c         If this is the first pass, don't
c         use derivative control.  Also set
c         the scale for total count.
c         ---------------------------------
c
                    if (gdrfl .lt. 0) then
                       azg = 0.35 * sens * azerr
                       elg = 0.35 * sens * elerr
c
                       gdrfl = - gdrfl
                       tcref = 1.33333 * fit (4)
c
c
c         Otherwise use derivative control.
c         ---------------------------------
c
                    else
                       azg = azerr + ((avps + 1) * (azerr - azerp))
                       azg = 0.35 * sens * azg
                       elg = elerr + ((avps + 1) * (elerr - elerp))
                       elg = 0.35 * sens * elg
c
                    endif
                    azerp = azerr
                    elerp = elerr
c
c         To correct the error, you must guide
c         in the other direction.  Duh.
c
                    azg = -1.0 * azg
                    elg = -1.0 * elg
c
c
c         The step really needs to end here
c         so the main program can call the
c         TCS communications handler.
c         ---------------------------------
c
                    if (abs (gdrfl) .eq. 3) gflag = .true.
                endif
c
             else
             endif
c
             step = 3
c
          endif
c
c
c         If a frame is being transmitted
c         via TCP/IP, set SENDCT to zero, to
c         indicate that the data is ready.
c         ----------------------------------
c
          if (sndptr .gt. 0) then
             sendct = 0
             sendfl = .false.
             sndptr = 0
          endif
c
c
c         The sky frame is complete.
c         --------------------------
c
          if (skyptr .gt. 0) then
             call sbox (2, 1)
             skyptr = 0
          endif
c
c
c         Format the magnified image,
c         and send it to the screen.
c         ---------------------------
c
       elseif (step .eq. 3) then
          if (imgtyp .gt. 0) then
             if     (imgtyp .eq. 2) then
                call magnif2
             elseif (imgtyp .eq. 4) then
                call magnif4
             endif
c
             step = 4
c
          else
             if (imgtyp .eq. -2) then
                call cbox (1, fit (1))
                call cbox (2, xerr)
                call cbox (3, yerr)
                call cbox (4, fit (4) / 4.)
                call cbox (5, fit (5) * px)
                call cbox (6, fit (6))
                call cbox (7, it)
                call cbox (20, fit (4) / tcref)
                call cbox (30, 0.5 * fit (5) * px)
                call cbox (40, azerr)
                call cbox (50, elerr)
c
             else
             endif
c
c
c
c         Put in the guider box.
c         ----------------------
c
             call boxwrt (magbuf)
c
c
c         Send the subraster image to the screen.
c         ---------------------------------------
c
             scrptr = 1
             do 380 iy = 9, 79
                call opqwrt (magbuf (scrptr), 71, 224, iy)
                scrptr = scrptr + 75
  380        continue
             step = 8
c
          endif
c
c
c         Put in the Z-values at the
c         locations of the 4 cursors.
c         ---------------------------
c
       elseif (step .eq. 4) then
          call curval (1)
          step = 5
c
       elseif (step .eq. 5) then
          call curval (2)
          step = 6
c
       elseif (step .eq. 6) then
          call curval (3)
          step = 7

       elseif (step .eq. 7) then
          call curval (4)
          step = 8
c
c
c         Set the zero point so pctval is
c         1/4 of span above zero.
c
c         Display the %tile value.
c         ------------------------
c
       elseif (step .eq. 8) then
ccc          if (pctset .gt. 0) then
cccccc             zzero = ((4 * pctval) - zspan) / 4
ccc             zzero = ((64 * pctval) / bkgset) - zspan
ccc             zzero = (bkgset * zzero) / 64
             call vbox (57, pctval)
ccc          else
ccc             zzero = zero
ccc          endif
c
c
c         Calculate the positions of the L's
c         --------------------------------
c
                radpa = abs (pa) / 57.29577951
                sinpa = sin (radpa)
                cospa = cos (radpa)
                call opqvec (elazx, elazy, elazx+eldx, elazy+eldy, bkg)
                call opqvec (elazx, elazy, elazx+azdx, elazy+azdy, bkg)
                call opqvec (dcrax, dcray, dcrax+dcdx, dcray+dcdy, bkg)
                call opqvec (dcrax, dcray, dcrax+radx, dcray+rady, bkg)
                eldx = nint ( 16.0 * sinpa)
                eldy = nint ( 16.0 * cospa)
                azdx = nint (-8.0 * cospa)
                azdy = nint ( 8.0 * sinpa)
                if (pa .lt. 0.) then
                    eldx = -eldx
                    azdx = -azdx
                endif
                radpa = radpa - parall/57.29577951
                sinpa = sin (radpa)
                cospa = cos (radpa)
                dcdx = nint ( 16.0 * sinpa)
                dcdy = nint ( 16.0 * cospa)
                radx = nint ( 8.0 * cospa)
                rady = nint (-8.0 * sinpa)
                if (pa .lt. 0.) then
                    dcdx = -dcdx
                    radx = -radx
                endif
                call opqvec (elazx, elazy, elazx+eldx, elazy+eldy, red)
                call opqvec (elazx, elazy, elazx+azdx, elazy+azdy, red)
                call opqvec (dcrax, dcray, dcrax+dcdx, dcray+dcdy, grn)
                call opqvec (dcrax, dcray, dcrax+radx, dcray+rady, grn)
          step = 9
c
c
c         Make sure the readout is really
c         finished, then re-set IMGFLG.
c         -------------------------------
c
       elseif (step .eq. 9) then
          if (and (imgflg, x'0002') .ne. 0) then
             imgflg = and (imgflg, x'FFFD')
             step = 0
          endif
c
       endif
c
  900  continue
       return
       end
c
c
c         Depending on the binning, call
c         the appropriate subraster painter.
c         ----------------------------------
c
       subroutine magnif
c
       implicit  none
c
       integer*4 imgtyp
c
       common / typcom / imgtyp
c
       if     (imgtyp .eq. 2) then
          call magnif2
       elseif (imgtyp .eq. 4) then
          call magnif4
       endif
c
       return
       end
c
c
c         Format a 71 x 71 subraster at
c         full magnification and send it
c         to the screen.
c
c         This version is for binned-2
c         mode.
c         ------------------------------
c
       subroutine magnif2
c
       implicit  none
c
       integer*4 incur, xcur (5), ycur (5)
       integer*4 magptr, xptr, yptr, xp, yp
       integer*4 imgsiz, imgptr, imgflg, zero
       integer*4 imgbuf (300000), zzero, zspan, bkgset
       integer*4 skyval, skybuf (300000)
       integer*4 ix, iy, i4
       logical*4 skyflg, subflg
       integer*1 magbuf (6000)
c
       common / imgcom / imgsiz, imgptr, imgflg, imgbuf
       common / skycom / skyflg, subflg, skyval, skybuf
       common / cr1com / incur, xcur, ycur
       common / zptcom / zero, zspan
       common / zrocom / zzero, bkgset
       common / magcom / magbuf
c
c
c         Fill up the buffer.  The first
c         line of image data is not used.
c         -------------------------------
c
       magptr = 0
c
       yptr = (ycur (5) / 10) - 35
       if (yptr .lt. 0) yptr = yptr + 1000
c
       do 280 iy = 1, 71
          yp = 500 * ((yptr / 2) + 1)
          xptr = (xcur (5) / 10) - 35
          if (xptr .lt. 0) xptr = xptr + 1000
c
          do 260 ix = 1, 71
             xp = xptr / 2
             i4 = imgbuf (yp + xp + 1)
             if (subflg) i4 = i4 + skyval - skybuf (yp + xp + 1)
             i4 = (64 * (i4 - zzero)) / zspan
             i4 = max (i4, 0)
             i4 = min (i4, 63)
             magptr = magptr + 1
             magbuf (magptr) = i4 + 16
             xptr = xptr + 1
             if (xptr .gt. 999) xptr = xptr - 1000
  260     continue
c
          yptr = yptr + 1
          if (yptr .gt. 999) yptr = yptr - 1000
  280  continue
c
c
c         Send to screen.
c         ---------------
c
       magptr = 1
       do 380 iy = 9, 79
          call opqwrt (magbuf (magptr), 71, 224, iy)
          magptr = magptr + 71
  380  continue
c
       return
       end
c
c
c         Format a 71 x 71 subraster at
c         full magnification and send it
c         to the screen.
c
c         This version is for binned-4
c         mode.
c         ------------------------------
c
       subroutine magnif4
c
       implicit  none
c
       integer*4 incur, xcur (5), ycur (5)
       integer*4 magptr, xptr, yptr, xp, yp
       integer*4 imgsiz, imgptr, imgflg, zero
       integer*4 imgbuf (300000), zzero, zspan, bkgset
       integer*4 skyval, skybuf (300000)
       integer*4 ix, iy, i4
       logical*4 skyflg, subflg
       integer*1 magbuf (6000)
c
       common / imgcom / imgsiz, imgptr, imgflg, imgbuf
       common / skycom / skyflg, subflg, skyval, skybuf
       common / cr1com / incur, xcur, ycur
       common / zptcom / zero, zspan
       common / zrocom / zzero, bkgset
       common / magcom / magbuf
c
c
c         Fill up the buffer.  The first
c         line of image data is not used.
c
c         Start 35 pixels before the box
c         center.
c         -------------------------------
c
       magptr = 0
c
       yptr = (ycur (5) / 10) - 35
       if (yptr .lt. 0) yptr = yptr + 1000
c
       do 280 iy = 1, 71
          yp = 250 * ((yptr / 4) + 1)
          xptr = (xcur (5) / 10) - 35
          if (xptr .lt. 0) xptr = xptr + 1000
c
          do 260 ix = 1, 71
             xp = xptr / 4
             i4 = imgbuf (yp + xp + 1)
             if (subflg) i4 = i4 + skyval - skybuf (yp + xp + 1)
             i4 = (64 * (i4 - zzero)) / zspan
             i4 = max (i4, 0)
             i4 = min (i4, 63)
             magptr = magptr + 1
             magbuf (magptr) = i4 + 16
             xptr = xptr + 1
             if (xptr .gt. 999) xptr = xptr - 1000
  260     continue
c
          yptr = yptr + 1
          if (yptr .gt. 999) yptr = yptr - 1000
  280  continue
c
c
c         Send to screen.
c         ---------------
c
       magptr = 1
       do 380 iy = 9, 79
          call opqwrt (magbuf (magptr), 71, 224, iy)
          magptr = magptr + 71
  380  continue
c
       return
       end
c
c
c         Write the Z-value at the location
c         of a cursor to the screen.
c         ---------------------------------
c
       subroutine curval (ic)
c
       implicit  none
c
       integer*4 ic, iptr, i4
       integer*4 incur, xcur (5), ycur (5)
       integer*4 imgsiz, imgptr, imgflg
       integer*4 imgbuf (300000)
c
       common / imgcom / imgsiz, imgptr, imgflg, imgbuf
       common / cr1com / incur, xcur, ycur
c
       if (imgsiz .lt. 80000) then
          iptr = 250 * (ycur (ic) / 40)
          iptr = iptr + 251 + (xcur (ic) / 40)
          i4 = imgbuf (iptr)
       else
          iptr = 500 * (ycur (ic) / 20)
          iptr = iptr + 501 + (xcur (ic) / 20)
          i4 = imgbuf (iptr)
       endif
c
       call vbox (40 + ic, i4)
c
       return
       end
