c
c
c         UT/ST clock program for Magellan guider
c         camera.
c
c         Time is kept as an r*8 variable which is
c         the universal time in units of 1 day
c         (utime), together with an i*4 variable
c         which is the modified julian day number
c         (mjday).
c
c         The clock interrupt service routine adds 1
c         to the number of clock ticks and calculates
c         the UT based on a value for the interval
c         between ticks (tstep), together with the
c         number of ticks (clkct) since the time the
c         clock was set (tzero).
c
c         Tstep is calculated from a specified clock
c         rate (ppm) relative to a nominal value.
c         ------------------------------------------
c
       subroutine utclk
c
       implicit none
c
       real*8     utime, tzset, tsset, utset
       real*4     etime
       integer*4  mjday, clkfl, ncyc, ppm, savct
       integer*4  timer1, timer2, timer3, timer4
       integer*4  time0, tprev, dtime, tmax, i4
       logical*4  setfl
c
       common / clkcom / utime, mjday
       common / clkvar / tzset, tsset, setfl, savct
       common / clkctl / utset, clkfl, ppm
       common / timcom / timer1, timer2, timer3, timer4
c
c
c
c         Set time for CPU clock.
c
c         If the clock interrupt turns over
c         to the next day at just the wrong
c         instant, then funny things might
c         happen?
c         ---------------------------------
c
       if (clkfl .eq. - 1) then
          clkfl = 1
          tsset = (1000000 - ppm) * 1.7165d-3 / 86400.d6
          tzset = utset
          setfl = .true.
       endif
c
c
c         Keep track of program cycle time.
c         First, see how many clock ticks have
c         elapsed.
c         ------------------------------------
c
       dtime = timer1 - time0
       if (dtime .lt. 0) dtime = dtime + 1000000000
c
c
c         600 clock ticks is just over 1 second.
c         First, calculate the total elapsed
c         program time.
c         Note: 50334984 = 24 * 3600 / .0017165
c         --------------------------------------
c
       if (dtime .ge. 600) then
          etime = timer1 / 50334984.
          etime = etime + timer2 / .050334984
          call mbox (24, etime)
c
c
c         Now calculate tha average cycle time
c         and the longest cycle time for the
c         previous 600 clock ticks.
c         ------------------------------------
c
          i4 = nint (1.7165 * dtime / ncyc)
          call pbox (3, i4)
          i4 = nint (1.7165 * tmax)
          call pbox (4, i4)
c
c
c         Reset the program cycle counter,
c         maximum cycle time, and start time.
c         -----------------------------------
c
          ncyc  = 0
          tmax  = 0
          time0 = timer1
          tprev = timer1
c
c
c         Otherwise, keep track of the number
c         of program cycles and the max cycle
c         time.
c         -----------------------------------
c
       else
          dtime = timer1 - tprev
          if (dtime .lt. 0) dtime = dtime + 1000000000
          if (dtime .gt. tmax) tmax = dtime
          tprev = timer1
       endif
       ncyc = ncyc + 1
c
c
c         Display universal time.
c         -----------------------
c
       call mbox (23, utime)
c
       return
       end
c