c
c
c         Draw a CCW circular arc, with
c         theta (deg) = 0 at the right.
c            center   = (x0, y0)
c            radius   = r
c            th1, th2 = angle endpoints
c            val      = screen color
c         -----------------------------
c
       subroutine arc (x0, y0, r, th1, th2, val)
c
       implicit none
c
       real*4    x0, y0, r, th1, th2, th, thf
       integer*4 val, vbuf (1200), ibuf
c
       th  = th1 / 57.29578
       thf = th2 / 57.29578
       vbuf (1) = nint (x0 + (r * cos (th)))
       vbuf (2) = nint (y0 - (r * sin (th)))
c
  100  continue
       th  = min (th + 0.01, thf)
       vbuf (3) = nint (x0 + (r * cos (th)))
       vbuf (4) = nint (y0 - (r * sin (th)))
       ibuf = 1
       call ovlvec (val, vbuf, ibuf)
c
       vbuf (1) = vbuf (3)
       vbuf (2) = vbuf (4)
       if (th .lt. thf) go to 100
c
       return
       end
c
c
c         draw a clock hand, with CCW
c         theta (rad) = 0 at the top.
c              center = (x0, y0)
c              length = + r1, - r2
c              angle  = th
c              width  = width
c
c              val    = screen color
c              vbuf   = storage for overwrite
c              ibuf   = pointer in vbuf
c         -----------------------------------
c
       integer*4 function handchk (x0, y0, r1, r2, th, width)
c
       implicit  none
c
       real*4    x0, y0, r1, r2, th, width
       real*4    x, y, hw, rr, sinth, costh
       integer*4 x1, y1, x2, y2, x3, y3
c
       common / handcom / x1, y1, x2, y2, x3, y3
c
       sinth = sin (th)
       costh = cos (th)
       hw    = 0.5 * width
       rr    = r2
c
       x = x0 + (hw * costh) + (rr * sinth)
       y = y0 - (hw * sinth) + (rr * costh)
       x1 = nint (x)
       y1 = nint (y)
       rr = rr + r1
       x  = x - (rr * sinth)
       y  = y - (rr * costh)
       x2 = nint (x)
       y2 = nint (y)
       x  = x2 - (width * costh)
       y  = y2 + (width * sinth)
       x3 = nint (x)
       y3 = nint (y)
c
       handchk = x2 + (1000 * y2)
       return
       end
c
c
c         second part of hand routine
c         ---------------------------
c
       subroutine hand (val, vbuf, ibuf)
c
       implicit  none
c
       integer*4 val, vbuf (1), ibuf, isav
       integer*4 x1, y1, x2, y2, x3, y3, x4, y4
c
       common / handcom / x1, y1, x2, y2, x3, y3
c
       x4 = x1 + x3 - x2
       y4 = y1 + y3 - y2
c
       isav = ibuf
       vbuf (ibuf)     =  1
       vbuf (ibuf + 1) = x1
       vbuf (ibuf + 2) = y1
       vbuf (ibuf + 3) = x2
       vbuf (ibuf + 4) = y2
       ibuf = ibuf + 1
       call ovlvec (val, vbuf, ibuf)
       vbuf (ibuf) = isav
       ibuf = ibuf + 1
c
       isav = ibuf
       vbuf (ibuf)     =  2
       vbuf (ibuf + 1) = x2
       vbuf (ibuf + 2) = y2
       vbuf (ibuf + 3) = x3
       vbuf (ibuf + 4) = y3
       ibuf = ibuf + 1
       call ovlvec (val, vbuf, ibuf)
       vbuf (ibuf) = isav
       ibuf = ibuf + 1
c
       isav = ibuf
       vbuf (ibuf)     =  3
       vbuf (ibuf + 1) = x3
       vbuf (ibuf + 2) = y3
       vbuf (ibuf + 3) = x4
       vbuf (ibuf + 4) = y4
       ibuf = ibuf + 1
       call ovlvec (val, vbuf, ibuf)
       vbuf (ibuf) = isav
       ibuf = ibuf + 1
c
       isav = ibuf
       vbuf (ibuf)     =  4
       vbuf (ibuf + 1) = x4
       vbuf (ibuf + 2) = y4
       vbuf (ibuf + 3) = x1
       vbuf (ibuf + 4) = y1
       ibuf = ibuf + 1
       call ovlvec (val, vbuf, ibuf)
       vbuf (ibuf) = isav
       ibuf = ibuf + 1
c
       return
       end
c
c
c         erase a clock hand
c         ------------------
c
       subroutine unhand (vbuf, ibuf)
c
       implicit  none
c
       integer*4 vbuf (1), ibuf, isav
c
  100  continue
       isav = vbuf (ibuf - 1)
       ibuf = isav + 1
       call ovlvec (-1, vbuf, ibuf)
       ibuf = isav
       if (vbuf (ibuf) .gt. 1) go to 100
c
       return
       end
c
c
c  * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
c  *                                                                     *
c  *   vector                  S. Shectman               July 11, 1995   *
c  *   ---------------------------------------------------------------   *
c  *        Draw an overlaying vector on the VGA screen.                 *
c  *        This version for Tseng Labs 256-color.                       *
c  *        The origin of the screen coordinate system is the            *
c  *           upper left-hand corner.                                   *
c  *        The x- and y- coordinates must be inside the                 *
c  *           plotting window: 0 <= x <= (lx - 1) and                   *
c  *           0 <= y <= (ly - 1).  No checking.                         *
c  *                                                                     *
c  *        arguments:                                                   *
c  *             x1, y1    ...... starting point of vector               *
c  *                              type = integer*4                       *
c  *                              stored in vbuf (ibuf, ibuf + 1)        *
c  *                                                                     *
c  *             x2, y2    ...... ending point of vector                 *
c  *                              type = integer*4                       *
c  *                              stored in vbuf (ibuf + 2, ibuf + 3)    *
c  *                                                                     *
c  *             draw      ...... draw = -1, write vector buf            *
c  *                              draw =  0... 255, read vector          *
c  *                                         buf and draw value          *
c  *                              type = integer*4                       *
c  *                                                                     *
c  *             vbuf      ...... buffer for vector screen values        *
c  *                              type = integer*4                       *
c  *                                                                     *
c  *             ibuf      ...... pointer to vector buffer               *
c  *                              points to (end + 1) on return          *
c  *                              type = integer*4                       *
c  *                                                                     *
c  * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
c
       subroutine ovlvec (draw, vbuf, ibuf)
c
       implicit none
c
       integer*4  x1, x2, y1, y2, vbuf (1), ibuf
       integer*4  x, xmax, y, ymax, inc, draw
       integer*4  d, d1, d2, dx, dy, peekb
       integer*4  page, npage, lpage, pgadr, ipg
       integer*4  opg, dpg, lx, ly, lxy
       integer*2  i2 (2)
c
       common / scpcom / opg, dpg, lx, ly, lxy
c
       equivalence (pgadr, i2)
c
       data  page / x 'A0000' /, npage / x'B0000' /
       data  lpage / x'10000' /
c
c
c         get arguments
c         -------------
c
       x1 = vbuf (ibuf)
       y1 = vbuf (ibuf + 1)
       x2 = vbuf (ibuf + 2)
       y2 = vbuf (ibuf + 3)
       ibuf = ibuf + 4
c
       dx = abs (x2 - x1)
       dy = abs (y2 - y1)
c
c
c         use bresenham's algorithm.
c         separate the cases for x-
c         and y- independent variable
c         ---------------------------
c
       if (dx .gt. dy)  then
          if (x2 .ge. x1)  then
             x = x1
             xmax = x2
             y = y1
             inc = sign (lx, y2 - y1)
          else
             x = x2
             xmax = x1
             y = y2
             inc = sign (lx, y1 - y2)
          endif
          pgadr = x + (lx * y)
          ipg = 17 * i2 (2)
          i2 (2) = 10
          call outp (x'3CD', ipg)
          d1 = 2 * dy
          d2 = 2 * (dy - dx)
          d  = d1 - dx
c
   20     continue
          if (draw .eq. -1) then
             call pokeb (pgadr, vbuf (ibuf))
          else
             vbuf (ibuf) = peekb (pgadr)
             call pokeb (pgadr, draw)
          endif
          ibuf = ibuf + 1
          if (x .ge. xmax)  RETURN
          x = x + 1
          pgadr = pgadr + 1
          if (d .lt. 0) then
             d = d + d1
          else
             pgadr = pgadr + inc
             d = d + d2
          endif
          if (pgadr .lt. page) then
             pgadr = pgadr + lpage
             ipg = ipg - 17
             call outp (x'3CD', ipg)
          elseif (pgadr .ge. npage) then
             pgadr = pgadr - lpage
             ipg = ipg + 17
             call outp (x'3CD', ipg)
          endif
          go to 20
c
       else
          if (y2 .ge. y1)  then
             y = y1
             ymax = y2
             x = x1
             inc = sign (1, x2 - x1)
          else
             y = y2
             ymax = y1
             x = x2
             inc = sign (1, x1 - x2)
          endif
          pgadr = x + (lx * y)
          ipg = 17 * i2 (2)
          i2 (2) = 10
          call outp (x'3CD', ipg)
          d1 = 2 * dx
          d2 = 2 * (dx - dy)
          d  = d1 - dy
c
   40     continue
          if (draw .eq. -1) then
             call pokeb (pgadr, vbuf (ibuf))
          else
             vbuf (ibuf) = peekb (pgadr)
             call pokeb (pgadr, draw)
          endif
          ibuf = ibuf + 1
          if (y .ge. ymax)  RETURN
          y = y + 1
          pgadr = pgadr - lx
          if (d .lt. 0) then
             d = d + d1
          else
             pgadr = pgadr + inc
             d = d + d2
          endif
          if (pgadr .lt. page) then
             pgadr = pgadr + lpage
             ipg = ipg - dpg
             call outp (x'3CD', ipg)
          elseif (pgadr .ge. npage) then
             pgadr = pgadr - lpage
             ipg = ipg + dpg
             call outp (x'3CD', ipg)
          endif
          go to 40
       endif
c
       end
c
c  * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
c  *                                                                     *
c  *   opqvec                  S. Shectman                July 7, 1996   *
c  *   ---------------------------------------------------------------   *
c  *        Draw a vector on the VGA screen.                             *
c  *        This version for Tseng Labs, 256-color.                      *
c  *        The origin of the screen coordinate system is the            *
c  *           upper left-hand corner.                                   *
c  *        The x- and y- coordinates must be inside the                 *
c  *           plotting window: 0 <= x <= (lx - 1) and                   *
c  *           0 <= y <= (ly - 1).  No checking.                         *
c  *                                                                     *
c  *        arguments:                                                   *
c  *             x1, y1    ...... starting point of vector               *
c  *                              type = integer*4                       *
c  *                                                                     *
c  *             x2, y2    ...... ending point of vector                 *
c  *                              type = integer*4                       *
c  *                                                                     *
c  *             draw      ...... 0... 255, color value                  *
c  *                              type = integer*4                       *
c  *                                                                     *
c  * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
c
       subroutine opqvec (x1, y1, x2, y2, draw)
c
       implicit none
c
       integer*4  x1, x2, y1, y2, draw
       integer*4  x, xmax, y, ymax, inc
       integer*4  d, d1, d2, dx, dy, peekb
       integer*4  page, npage, lpage, pgadr, ipg
       integer*4  opg, dpg, lx, ly, lxy
       integer*2  i2 (2)
c
       common / scpcom / opg, dpg, lx, ly, lxy
c
       equivalence (pgadr, i2)
c
       data  page / x 'A0000' /, npage / x'B0000' /
       data  lpage / x'10000' /
c
       dx = abs (x2 - x1)
       dy = abs (y2 - y1)
c
c
c         use bresenham's algorithm.
c         separate the cases for x-
c         and y- independent variable
c         ---------------------------
c
       if (dx .gt. dy)  then
          if (x2 .ge. x1)  then
             x = x1
             xmax = x2
             y = y1
             inc = sign (lx, y2 - y1)
          else
             x = x2
             xmax = x1
             y = y2
             inc = sign (lx, y1 - y2)
          endif
          pgadr = x + (lx * y)
          ipg = i2 (2)
          i2 (2) = 10
          call outp (x'3CD', ipg)
          d1 = 2 * dy
          d2 = 2 * (dy - dx)
          d  = d1 - dx
c
   20     continue
          call pokeb (pgadr, draw)
          if (x .ge. xmax)  RETURN
          x = x + 1
          pgadr = pgadr + 1
          if (d .lt. 0) then
             d = d + d1
          else
             pgadr = pgadr + inc
             d = d + d2
          endif
          if (pgadr .lt. page) then
             pgadr = pgadr + lpage
             ipg = ipg - 1
             call outp (x'3CD', ipg)
          elseif (pgadr .ge. npage) then
             pgadr = pgadr - lpage
             ipg = ipg + 1
             call outp (x'3CD', ipg)
          endif
          go to 20
c
       else
          if (y2 .ge. y1)  then
             y = y1
             ymax = y2
             x = x1
             inc = sign (1, x2 - x1)
          else
             y = y2
             ymax = y1
             x = x2
             inc = sign (1, x1 - x2)
          endif
          pgadr = x + (lx * y)
          ipg = i2 (2)
          i2 (2) = 10
          call outp (x'3CD', ipg)
          d1 = 2 * dx
          d2 = 2 * (dx - dy)
          d  = d1 - dy
c
   40     continue
          call pokeb (pgadr, draw)
          if (y .ge. ymax)  RETURN
          y = y + 1
          pgadr = pgadr + lx
          if (d .lt. 0) then
             d = d + d1
          else
             pgadr = pgadr + inc
             d = d + d2
          endif
          if (pgadr .lt. page) then
             pgadr = pgadr + lpage
             ipg = ipg - 1
             call outp (x'3CD', ipg)
          elseif (pgadr .ge. npage) then
             pgadr = pgadr - lpage
             ipg = ipg + 1
             call outp (x'3CD', ipg)
          endif
          go to 40
       endif
c
       end
c
